#include <stdint.h>

#include "debug.h"
#include "drivers.h"
#include "interrupts.h"
#include "kprintf.h"
#include "memory.h"
#include "sys/io.h"
#include "task.h"

#define PIT_CHANNEL0  0x40
#define PIT_FREQUENCY 1193182
#define PIT_COUNT     (65535 / 2)
#define DELAY         (1000 / (PIT_FREQUENCY / PIT_COUNT))

static uint32_t sleep_counter;
static uint32_t scheduler_counter;

static void clock_handler(struct registers *regs);

static void set_pit_count(unsigned count)
{
	cli();

	outb(0x40, count & 0xFF);          // Low byte
	outb(0x40, (count & 0xFF00) >> 8); // High byte

	toris();
}

void clock_init(struct registers *regs)
{
	(void)regs;
	set_pit_count(PIT_COUNT);
	register_interrupt_handler(0, clock_handler);
}

static void clock_handler(struct registers *regs)
{
	(void)regs;
	if (scheduler_counter % 10 == 0)
		scheduler();
	scheduler_counter++;
	sleep_counter--;
}

void sleep(uint64_t delay)
{
	sleep_counter = delay / DELAY;
	while (sleep_counter)
		;
}
