#include "alloc.h"
#include "string.h"
#include <stdint.h>

// Prototype for kfree and kmalloc
void kfree(void *ptr);
void *kmalloc(uint32_t size);

/*
 * ptr: block to resize (undefined behavior if invalid)
 * size: size needed by the user to get kreallocated
 *
 * If we have a size <= to the previous size, we don't have
 * to do anything, we just change sub_size for info purposes
 * and return the same pointer
 * Else, we allocate a new block and copy the content of
 * the previous block in the new one and kfree the old block
 *
 * ptr: returns the aligned pointer of the kreallocated block
 */
void *krealloc(void *ptr, uint32_t size)
{
	void *new_ptr = NULL;
	if (ptr == NULL)
		return NULL;
	Block *block = (Block *)((uint32_t)ptr - sizeof(Block));
	if (block->size >= size) {
		block->sub_size = size;
		return (ptr);
	}
	new_ptr = kmalloc(size);
	if (new_ptr == NULL)
		return NULL;
	memmove(new_ptr, ptr, block->size);
	kfree(ptr);
	return (new_ptr);
}
