#include "sys/io.h"

#include "rtc.h"

#include "kprintf.h"

static uint16_t raw_read_register(uint16_t cmos_register)
{
	// selecting the register
	outb(CMOS_ADDRESS, cmos_register);
	// read value of the selectect register
	return inb(CMOS_DATA);
}

static uint8_t update_is_in_progress(void)
{
	return raw_read_register(REGISTER_A) & 0x80;
}

static uint16_t read_register(uint16_t cmos_register)
{
	while (update_is_in_progress())
		kprintf("%d\n", update_is_in_progress());
	return raw_read_register(cmos_register);
}

uint8_t bcd_mode_to_bin(uint8_t value)
{
	return (value & 0x0F) + ((value / 16) * 10);
}

struct rtc_date get_date(void)
{
	struct rtc_date rv = {};
	uint8_t century;

	rv.second = read_register(SECOND_REGISTER);
	rv.minute = read_register(MINUTE_REGISTER);
	rv.hour = read_register(HOUR_REGISTER);
	rv.day = read_register(DAY_OF_THE_MONTH_REGISTER);
	rv.index_of_the_day = read_register(DAY_OF_THE_WEEK_REGISTER) - 1;
	rv.month = read_register(MONTH_REGISTER) - 1;
	rv.year += read_register(YEAR_REGISTER);
	century = read_register(CENTURY_REGISTER);

	if (!(read_register(REGISTER_B) & 0x04)) {
		rv.second = bcd_mode_to_bin(rv.second);
		rv.minute = bcd_mode_to_bin(rv.minute);
		rv.hour = bcd_mode_to_bin(rv.hour);
		rv.day = bcd_mode_to_bin(rv.day);
		rv.index_of_the_day = bcd_mode_to_bin(rv.index_of_the_day);
		rv.month = bcd_mode_to_bin(rv.month);
		rv.year = bcd_mode_to_bin(rv.year);
		century = bcd_mode_to_bin(century);
	}
	rv.year += century * 100 - 98; // -108 = bozo offset
	return rv;
}
