#include "signal.h"
#include "kprintf.h"
#include "task.h"
#include <stdint.h>

void signal(SIGNAL_CODE sig_num, void *handler)
{
	current_task->signals.handlers[sig_num] = handler;
}

void kill(int pid, SIGNAL_CODE sig_num)
{
	struct task *task = current_task;

	while (task->pid != pid)
		task = task->next;

	task->signals.pending = sig_num;
}

static void display_signal(SIGNAL_CODE sig_num)
{
	kprintf("signal: %d\n", sig_num);
}

static void exec_signal(SIGNAL_CODE sig_num)
{
	void *handler = current_task->signals.handlers[sig_num];

	if (handler == SIG_IGN)
		return;

	if (handler == SIG_DFL) {
		display_signal(sig_num);
		return;
	}

	signal_handler_t func = handler;
	func(sig_num);
}

void exec_signal_pending(void)
{
	uint32_t signal_code = current_task->signals.pending;
	if (signal_code != SIG_IGN) {
		exec_signal(signal_code);
		current_task->signals.pending = SIG_IGN;
	}
}